# Define the file paths
thingy_table_path = "Puyo Puyo Fever.tbl"
text_file_path = "text_file.txt"
output_file_path = "hex_file.bin"

# Parse the Thingy table into a dictionary
thingy_dict = {}
with open(thingy_table_path, "r", encoding="utf-8") as thingy_table:
    for line in thingy_table:
        if line.startswith("#"):
            continue
        key, value = line.strip().split("=", 1)
        thingy_dict[value] = key

# Parse the text file and create the hex file
byte_counts = []
with open(text_file_path, "r", encoding="utf-8") as text_file, \
        open(output_file_path, "wb") as hex_file:
    for line_num, line in enumerate(text_file, start=1):
        byte_count = 0
        i = 0
        while i < len(line):
            char = line[i]
            if char == " ":
                hex_value = "0080"
            elif char == "$":
                hex_value = line[i+1:i+5]
                i += 4
            elif char == "[":
                control_code = ""
                for control_char in line[i+1:]:
                    if control_char == "]":
                        break
                    control_code += control_char
                control_code = "[" + control_code + "]"
                hex_value = thingy_dict[control_code]
                i += len(control_code) - 1
            elif char == "\\":
                next_char = line[i+1]
                if next_char in ["$", "[", "]"]:
                    control_code = "\\" + next_char
                    hex_value = thingy_dict[control_code]
                    i += 1
                else:
                    hex_value = ""
            elif char in thingy_dict:
                hex_value = thingy_dict[char]
            else:
                hex_value = ""
            if hex_value:
                hex_file.write(bytes.fromhex(hex_value))
                byte_count += 2
            i += 1
        byte_counts.append(byte_count)
        print(f"Line {line_num}: {byte_count//2} bytes written")

# Get the total number of bytes and number of lines in the text file
total_bytes = sum(byte_counts)
num_lines = len(byte_counts)

# Generate the header data
header_data = []
header_data.append((4 + total_bytes + 4*num_lines) // 2) # Total number of bytes divided by 2
header_data.append(2 + 2*num_lines) # Length of header divided by 2
for i in range(num_lines):
    line_length = byte_counts[i] // 2
    if i == 0:
        header_data.append(0)
    else:
        header_data.append(header_data[-1] + header_data[-2])
    header_data.append(line_length)

# Temporarily save the data of the hex file
with open(output_file_path, "rb") as hex_file:
    hex_data = hex_file.read()

# Overwrite the hex file with the header and hex data
with open(output_file_path, "wb") as hex_file:
    # Write header
    for value in header_data:
        hex_file.write(value.to_bytes(2, byteorder='little'))
    # Write hex data
    hex_file.write(hex_data)
